# -*- coding: utf-8 -*-
"""
Created on Tue Jun  5 12:27:49 2018

@author: imarshal
"""

import unittest
import cards

class TestCases(unittest.TestCase):

    def test_std_deck_as_text(self):
        d1 = cards.Deck()
        s = ("1D | 2D | 3D | 4D | 5D | 6D | 7D | 8D | 9D | 10D | 11D | 12D | 13D | "
             "1C | 2C | 3C | 4C | 5C | 6C | 7C | 8C | 9C | 10C | 11C | 12C | 13C | "
             "1H | 2H | 3H | 4H | 5H | 6H | 7H | 8H | 9H | 10H | 11H | 12H | 13H | "
             "1S | 2S | 3S | 4S | 5S | 6S | 7S | 8S | 9S | 10S | 11S | 12S | 13S")
        self.assertEqual(d1.deck_as_text(),s, "Standard deck doesn't print correctly")
    def test_deck_comparison(self):
        d1 = cards.Deck()
        d2 = cards.Deck()
        self.assertEqual(d1, d2, "Deck comparison operator fails on 2 standard decks")
    def test_shuffle_full_deck(self):
        d1 = cards.Deck()
        d2 = cards.Deck()
        d2.shuffle_deck()
        self.assertNotEqual(d1, d2, "Shuffled deck did not change") #might not be the perfect test case,
        #but statistically nearly impossible to shuffle deck back order
    def test_shuffle_then_order_full_deck(self):
        d1 = cards.Deck()
        d2 = cards.Deck()
        d2.shuffle_deck()
        d2.order_deck()
        self.assertEqual(d1, d2, "Shuffled deck did not reorder on order() call")
    def test_get_suit_from_std_deck(self):
        d1 = cards.Deck()
        diamond_string = "1D | 2D | 3D | 4D | 5D | 6D | 7D | 8D | 9D | 10D | 11D | 12D | 13D"
        club_string = "1C | 2C | 3C | 4C | 5C | 6C | 7C | 8C | 9C | 10C | 11C | 12C | 13C"
        heart_string = "1H | 2H | 3H | 4H | 5H | 6H | 7H | 8H | 9H | 10H | 11H | 12H | 13H"
        spade_string = "1S | 2S | 3S | 4S | 5S | 6S | 7S | 8S | 9S | 10S | 11S | 12S | 13S"
        diamond_deck = d1.get_suit("Diamond")
        club_deck = d1.get_suit("Club")
        heart_deck = d1.get_suit("Heart")
        spade_deck = d1.get_suit("Spade")
        self.assertEqual(diamond_deck.deck_as_text(),diamond_string, "get_suit() failed Diamond")
        self.assertEqual(club_deck.deck_as_text(),club_string, "get_suit() failed Club")
        self.assertEqual(heart_deck.deck_as_text(),heart_string, "get_suit() failed Heart")
        self.assertEqual(spade_deck.deck_as_text(),spade_string, "get_suit() failed Spade")



if __name__ == '__main__':
    unittest.main()


# -*- coding: utf-8 -*-
"""
Created on Mon Jun  4 15:28:08 2018

@author: imarshal
"""
import random
suits = { "Diamond" : 0, "Club" : 1, "Heart" : 2 , "Spade" : 3 }
suit_names = ["Diamond", "Club", "Heart", "Spade"]

class Card():
    def __init__(self, value0, suit0):
        if (suit0 in suit_names and value0 in range(1, 14)):
            self.suit = suit0
            self.value = value0
            self.order = suits[suit0]*13+value0
        else:
            1 == 1
            #invalid card
    #overload comaprison operators for easier sorting
    def __eq__(self, other):
        return (self.order) == other.order                     
    def __lt__(self, other):
        return (self.order) < other.order
    def __gt__(self, other):
        return (self.order) > other.order
    def __le__(self, other):
        return (self.order) <= other.order
    def __ge__(self, other):
        return (self.order) >= other.order


class Deck():
    def __init__(self, std_deck=True):
        ###default initializes in order deck
        self.cards = []
        if std_deck:
            for s in suit_names:
                for v in range(1, 14):
                    self.cards.append(Card(v, s))
    
    def __eq__(self, other):
        return (self.cards) == other.cards  

    def deck_as_text(self):
        deck_string = ''
        for c in self.cards:
            deck_string = deck_string + str(c.value) + c.suit[0] + " | "
        return deck_string[:-3]
    
    def shuffle_deck(self):
        random.shuffle(self.cards)
    
    def order_deck(self): #python 
        (self.cards).sort()

    def get_suit(self, suit0):
        d0 = Deck(std_deck=False)
        for c in self.cards:
            if c.suit == suit0:
                d0.cards.append(c)
        return d0
            
    def quick_sort_deck(self):
        #Define Helpers so we can pass only the Deck
        def quick_sort_helper(card_list, first, last):
            if first<last:
               splitpoint = partition(card_list,first,last)
               quick_sort_helper(card_list,first,splitpoint-1)
               quick_sort_helper(card_list,splitpoint+1,last)
    
        def partition(card_list, first, last):
            pivotvalue = card_list[first]
            left = first+1
            right = last
            while True:
                while left <= right and card_list[left] <= pivotvalue:
                    left = left + 1
    
                while card_list[right] >= pivotvalue and right >= left:
                    right = right -1
                if right < left:
                    break #sub list sorted
                else:
                    temp = card_list[left]
                    card_list[left] = card_list[right]
                    card_list[right] = temp
            temp = card_list[first]
            card_list[first] = card_list[right]
            card_list[right] = temp
            return right       
        card_list = self.cards
        quick_sort_helper(card_list,0,len(card_list)-1)
        self.cards = card_list
       


        
# if __name__ == '__main__':
#     print suits
#     print suits.keys()
#     print suits.items()
    #c1 = Card(3, "Heart")
    #c1.print()
    #c1.verbose_print()
    # print("============Initialize Deck============")
    # d1 = Deck()
    # h1 = d1.get_suit("Spades")
    # print(d1.deck_as_text())
    # print("============Initialize Deck============")
    # pritn(h1.deck_as_text())
    # print("=============Shuffled===============")
    # d1.shuffle_deck()
    # h1 = d1.get_suit("Spades")
    # print(d1.deck_as_text())
    # print("==============Re ordered===============")
    # d1.order_deck()
    # print(d1.deck_as_text())
    # print("=============Shuffled Again===============")
    # d1.shuffle_deck()
    # print(d1.deck_as_text())
    # print("=============Quick Sort===============")
    # d1.quick_sort_deck()
    # print(d1.deck_as_text())


from setuptools import setup

setup(name='cards_imarshall',
      version='0.1',
      description='Python Card Deck Module',
      url='https://github.com/IanNMarshall/cards_imarshall',
      author='IanNMarshall',
      author_email='ian.nicholas.marshall@gmail.com',
      packages=['cards_imarshall'],
      zip_safe=False)
